#!/usr/local/bin/wish
set Revision {$Revision: 1.18 $ $Date: 2001/08/13 18:30:44 $}
#-----------------------------------------------------------
#-----------------------------------------------------------
# CMPR notes (to do)
# idea: bind <F1> for toplevels to generate the appropriate help
#-----------------------------------------------------------
#-----------------------------------------------------------
set command(debug) 0
catch {if $env(DEBUG) {set command(debug) 1}}
# where are we?
set scriptname [info script]
# translate links -- go six levels deep
foreach i {1 2 3 4 5 6} {
    if {[file type $scriptname] == "link"} {
	set link [file readlink $scriptname]
	if { [file  pathtype  $link] == "absolute" } {
	    set scriptname $link
	} {
	    set scriptname [file dirname $scriptname]/$link
	}
    } else {
	break
    }
}
# fixup relative paths
if {[file pathtype $scriptname] == "relative"} {
    set scriptname [file join [pwd] $scriptname]
}
set scriptdir [file dirname $scriptname]
#----------------------------------------------------------------

if [catch {package require BLT} errmsg] {
    tk_dialog .err "BLT Error" "Error -- Unable to load the BLT package" \
	    error 0 Quit
#    destroy .
    exit
}
# handle Tcl/Tk v8+ where BLT is in a namespace
#  use the command so that it is loaded
catch {blt::graph}
catch {
    namespace import blt::graph
    namespace import blt::vector
}
catch {graph}

# old versions of BLT (<=2.1) don't report a version number
if [catch {set blt_version}] {set blt_version 0}

# option for coloring markers: note that GH keeps changing how to do this!
if {$blt_version < 2.3 || $blt_version >= 8.0} {
    set graph(MarkerColorOpt) -fg 
} elseif {$blt_version >= 2.4} {
    set graph(MarkerColorOpt) -outline
} else {
    set graph(MarkerColorOpt) -color
}

lappend auto_path $scriptdir
set command(haveBW) 1
# for debugging non-BWidget version set environment variable NOBWIDGET
catch {if $env(NOBWIDGET) {set command(haveBW) 0}}
if $command(haveBW) {
    if [catch {package require BWidget}] {set command(haveBW) 0}
}

#----------------------------------------------------------------
# set default values for lots of variables
if {$tcl_platform(platform) == "windows"} {
    set cellparm(hklgenprog) "| [file join $scriptdir hklgen.exe]"
} else {
    set cellparm(hklgenprog) "| [file join $scriptdir hklgen] |& cat"
}
set cellparm(tstextprog) "| [file join $scriptdir testextn]"
set cellparm(calc2tprog) "| [file join $scriptdir calcpos]"
set cellparm(spacegroup) (none)
set cellparm(extspg) (none)
set cellparm(zero) 0
set cellparm(old_zero) 0
set cellparm(old_ymin) 0
set cellparm(old_ymax) 0
set cellparm(old_wave) -1
set cellparm(oldextspg) {xxx} 
set cellparm(sens) 25
set cellparm(outcount) 0
set cellparm(twothetamax) 168
set cellparm(wavelength) "1.5396"
set cellparm(alpha) [set cellparm(beta) [set cellparm(gamma) 90]]
set cellparm(a) [set cellparm(b) [set cellparm(c) 10]]
set cellparm(laue) 8
set cellparm(sens) 25
set cellparm(fadetime) 10
set cellparm(ymin) 0
set cellparm(ymax) 1000
set command(writeunits) 0
set command(readbuttons) {}
set command(readbuttonproc) {}
set command(helpmode) 1
# command(pagenow) is used to indicate which screen is currently shown
set command(pagenow) ""
set graph(outname) out.ps
set graph(outcmd) "lpr -h"
if {$tcl_platform(platform) == "windows"} {
    set graph(printout) 1
} else {
    set graph(printout) 0
}
set graph(legend) 0
set cellparm(fontsize) 15
# define a font used for labels
if {$tcl_version >= 8.0} {
  font create lblfont -family Helvetica -size [expr -$cellparm(fontsize)]
} 
set graph(units) 0
set menulist(options) {}

# called by a trace on cellparm(fontsize)   
proc setfontsize {a b c} {
    global cellparm graph
    catch {
	font config lblfont -size [expr -$cellparm(fontsize)]
	# this forces a redraw of the plot by changing the title to itself
	$graph(blt) configure -title [$graph(blt) cget -title]
    }
}
trace variable cellparm(fontsize) w setfontsize
#-------------------------------------------------------------------------
#
# Convenient "quit" key
#
bind all <Control-KeyPress-c> {Setup Quit}
focus .
vector xextinct 
vector yextinct 
set graph(datalist) {}
set graph(plotlist) {}
array set command  {xoffset 0 xmult 1 yoffset 0 ymult 1 choice ""}
#-------------------------------------------------------------------------
# create a window for the plot
catch {destroy .plot}
toplevel .plot
# build the graph
pack [set graph(blt) [graph .plot.gr]] -fill both -expand yes -side top
#-------------------------------------------------------------------------
# define a binding to show the cursor location
proc ToggleLiveCursor {} {
    if {[bind .plot <Any-Motion>] == ""} {
	pack [frame .plot.bot -bd 2 -relief sunken] -side bottom -fill x
	pack [label .plot.bot.val1 -textvariable graph(position)] -side left
	pack [button .plot.bot.close -command ToggleLiveCursor -text "Close cursor display"] -side right
	bind .plot <Any-Motion> {FormatLiveCursor %x %y}
    } else {
	destroy .plot.bot
	bind .plot <Any-Motion> {}
    }
}
proc FormatLiveCursor {x y} {
    global graph
    set graph(position) \
	    "x=[format %.3f [$graph(blt) xaxis invtransform $x]] y=[format %.3f [$graph(blt) yaxis invtransform $y]]"
}
bind .plot <Key-l> {ToggleLiveCursor}
bind .plot <Key-L> {ToggleLiveCursor}
#-------------------------------------------------------------------------
# manual zoom option
proc BLTmanualZoom {} {
    global graph
    catch {toplevel .zoom}
    eval destroy [grid slaves .zoom]
    raise .zoom
    wm title .zoom {Manual Scaling}
    grid [label .zoom.l1 -text minimum] -row 1 -column 2 
    grid [label .zoom.l2 -text maximum] -row 1 -column 3 
    grid [label .zoom.l3 -text x] -row 2 -column 1 
    grid [label .zoom.l4 -text y] -row 3 -column 1 
    grid [entry .zoom.xmin -textvariable graph(xmin) -width 10] -row 2 -column 2 
    grid [entry .zoom.xmax -textvariable graph(xmax) -width 10] -row 2 -column 3 
    grid [entry .zoom.ymin -textvariable graph(ymin) -width 10] -row 3 -column 2 
    grid [entry .zoom.ymax -textvariable graph(ymax) -width 10] -row 3 -column 3 
    grid [frame .zoom.b] -row 4 -column 1 -columnspan 3
    grid [button .zoom.b.1 -text "Set Scaling" \
	     -command "SetManualZoom set"]  -row 4 -column 1 -columnspan 2
    grid [button .zoom.b.2 -text Reset \
	    -command "SetManualZoom clear"] -row 4 -column 3
    grid [button .zoom.b.3 -text Close -command "destroy .zoom"] -row 4 -column 4 
    grid rowconfigure .zoom 1 -weight 1 -pad 5
    grid rowconfigure .zoom 2 -weight 1 -pad 5
    grid rowconfigure .zoom 3 -weight 1 -pad 5
    grid rowconfigure .zoom 4 -weight 0 -pad 5
    grid columnconfigure .zoom 1 -weight 1 -pad 20
    grid columnconfigure .zoom 1 -weight 1 
    grid columnconfigure .zoom 3 -weight 1 -pad 10
    foreach item {min min max max} \
	    format {3   2   3   2} \
	    axis   {x   y   x   y} {
	set val [$graph(blt) ${axis}axis cget -${item}]
	set graph(${axis}${item}) {(auto)}
	catch {set graph(${axis}${item}) [format %.${format}f $val]}
    }
}

proc SetManualZoom {mode} {
    global graph
    if {$mode == "clear"} {
	foreach item {xmin ymin xmax ymax} {
	    set graph($item) {(auto)}
	}
    }
    foreach item {xmin ymin xmax ymax} {
	set $item {}
	catch {set $item [expr $graph($item)]}	
    }
    # reset the zoomstack
    catch {Blt_ZoomStack $graph(blt)}
    catch {$graph(blt) xaxis config -min $xmin -max $xmax}
    catch {$graph(blt) yaxis config -min $ymin -max $ymax}
}
bind .plot <Key-z> {BLTmanualZoom}
bind .plot <Key-Z> {BLTmanualZoom}
set helplist(Shortcuts) {\
There are a number of shortcuts implemented by pressing 
keys or mouse buttons. Keys are not case-sensitive.

in plot window/any page
   L -- shows "live" cursor position
   Z -- set zoom scaling manually
   left click -- zoom in
   right click -- zoom out

in plot window/EditCell page only
   shift+left click -- label nearby reflections
   shift+double left click -- label all reflections
   shift+right click -- delete reflection labels

in plot window/fit page only
   control+double left click -- add peak at current position
   S -- set fit range to current zoom
   F -- scroll the fit region forward
   B -- scroll thefit region backward
   E -- expand the scroll fit region

in any window/any page:
  control+C -- exits the program
  F1 key    -- show help on current page

Also see File Selection Tricks
}

#-------------------------------------------------------------------------
if [file exists [file join $scriptdir opts.tcl]] {
    source [file join $scriptdir opts.tcl]
}
source [file join $scriptdir data.tcl]
source [file join $scriptdir read.tcl]
source [file join $scriptdir write.tcl]
source [file join $scriptdir symsubs.tcl]
source [file join $scriptdir hklgen.tcl]
source [file join $scriptdir fit.tcl]
source [file join $scriptdir index.tcl]
if {!$command(haveBW)} {source [file join $scriptdir notebook.tcl]}

# add any locally defined routines for reads
foreach file [lsort [glob -nocomplain [file join $scriptdir read_*.tcl]]] {
    source $file
}
# add any locally defined routines for writes
foreach file [lsort [glob -nocomplain [file join $scriptdir write_*.tcl]]] {
    source $file
}
# add any locally defined routines (for reads, etc.)
set file [file join $scriptdir localcmds.tcl]
if [file exists $file] {source $file}
# add any personalized defaults
if {[catch {
    if [file exists ~/.cmpr_init] {source ~/.cmpr_init}
} errmsg]} {
    tk_dialog .err "cmpr_init Error" \
	    "Error -- while processing .cmpr_init:\n $errmsg" \
	    error 0 "Limp Ahead"
}
   
#----------------------------------------------------------------
# and now on with the code
#----------------------------------------------------------------
proc DoCommand {cmd} {
    global command
    # draw the page
    if {[catch {Setup $cmd} error]} {
	if {$command(debug)} {puts $error}
    }
}

# post a command dialog page
proc Setup {cmd} {
    global command
    if {$cmd == "Quit"} {
	if {[tk_messageBox -message "Really quit?" -type yesno -icon question] \
		== "yes"} {exit}
    } elseif $command(haveBW) {
	set pagename [string tolower $cmd]
	$command(notebook) see $pagename
	$command(notebook) raise $pagename
    } else {
	Notebook:raise $command(notebook) $cmd
    }
    set command(pagenow) [string tolower $cmd]
}
# resize the notebook to fit all the tabs and the largest page
proc ResizeNotebook {} {
    global command
    if {$command(haveBW)} {
	$command(notebook) compute_size
    } else {
	Notebook:resize $command(notebook)
    }
}
#----------------------------------------------------------------
# final initializations
#----------------------------------------------------------------
# locations of the extinction condition sub-menus
set cellparm(extinctmenuH) {}
set cellparm(extinctmenuE) {}
resetextinctions  
trace var extinctlbl w extinctlblset 

#----------------------------------------------------------------
#Quit
lappend menulist(file) Quit 
set helplist(Quit) {Quit the program}
#----------------------------------------------------------------
#Plot
lappend menulist(display) Plot
lappend menulist(pages) Plot
set helplist(Plot) {\
The plot page is used to display one or more buffers. To select 
a single buffer, click on the buffer name.  A range of buffers 
can be selected by dragging with the mouse. Individual buffers 
can be selected or deselected by holding the control key and 
clicking on the buffer name. The plot is updated when the 
"Update Plot" button is pressed or with a mouse double-click.

The "Postscript out" button causes a postscript copy of the 
plot window to be created. Settings on the Options page 
determine where the file is created or how it is printed. 
Plots can be exported to, a publication-quality graphics package 
where the displayed plot can be enchanced and annotated.

This page is also used to change the way a file is displayed. 
Do this by selecting the file(s) to be changed and then using 
the appropriate checkboxes to select the color, line type, 
symbol type and size. Changes will be made when the "Apply 
Changes" button is pressed.
}

proc MakePlot {page} {
#    pack [frame $page.a] -side left -anchor nw -fill y
    grid [frame $page.a] -column 0 -row 0 -rowspan 2 -sticky nw
    pack [label $page.a.t1 -text {Select dataset(s) to}] -side top
    pack [label $page.a.t2 -text {plot or change...}] -side top
    pack [frame $page.a.2 ] -side top 
    makeselectbox $page.a.2
    pack [button $page.a.h -text "PostScript out" \
	    -command makepostscriptout] -side bottom
    pack [button $page.a.p -text "Update Plot" \
	    -command "getfilelist $page.a.2; displaylist"] -side bottom

#    pack [frame $page.c] -side left -anchor nw -fill both
#    pack [button $page.c.g -text "Export\nto Grace" \
#	    -command makeGraceExport] -side bottom
    grid [button $page.g -text "Export\nto Grace" \
	    -command makeGraceExport] -column 1 -row 1 -sticky sw
    grid columnconfigure $page 1 -weight 1
#    pack [frame $page.b -bd 4 -relief groove] -side top -anchor ne -fill y
    grid [frame $page.b -bd 4 -relief groove] \
	    -column 1 -columnspan 2 -row 0 -sticky ne
    pack [frame $page.b.1] -side top -anchor n -fill both
    pack [frame $page.b.2] -side top -anchor n -fill both
    pack [frame $page.b.1.b] -side left -anchor n -fill both
    pack [label $page.b.1.b.t -text Line] -side top 
    foreach symbol {"" 0 1 2 4} \
	    symbol_name {"no change" "no line" "thin line" "medium" "thick"} {
	pack [radiobutton $page.b.1.b.$symbol_name \
		-text $symbol_name -variable command(line) \
		-value $symbol] -side top -anchor w
    }

    pack [frame $page.b.1.c] -side left -anchor n
    pack [label $page.b.1.c.t -text Color] -side top 
    foreach color {"no change" black red green blue magenta cyan yellow} {
	pack [radiobutton $page.b.1.c.$color \
	    -text $color -variable command(color) -value $color \
	    ] -side top -anchor w
	if {$color != "no change"} {$page.b.1.c.$color config -fg $color}
    }
    pack [frame $page.b.1.d] -side left -anchor n
    pack [label $page.b.1.d.t -text "Symbol type"] -side top
    foreach symbol {"" none square circle diamond plus cross \
	    splus scross} \
	    symbol_name {"no change" none square circle diamond plus cross \
	    thin-plus thin-cross} {
	pack [radiobutton $page.b.1.d.$symbol \
	    -text $symbol_name -variable command(symbol) \
	    -value $symbol] -side top -anchor w
    }
    pack [frame $page.b.1.e] -side left -anchor n -fill y
    pack [label $page.b.1.e.l -text "Symbol Size"] -side top
    pack [scale $page.b.1.e.s -variable command(size) \
	    -from .1 -to 5 -resolution .1] -side top
    pack [button $page.b.2.b \
	    -text "Apply Changes" \
	    -command "ChangeColors $page.a.2"] -side bottom
}

proc PostPagePlot {page} {
    global command
    set command(line) ""
    set command(symbol) ""
    set command(color) "no change"
    set command(size) 1
    updateselectbox $page.a.2
}

proc ChangeColors {box} {
    global command
    foreach num [$box.1 curselection ] {
	set data [$box.1 get $num]
	global $data
	if {$command(line) != ""} {set ${data}(line) $command(line)}
	if {$command(color) != "no change"} {
	    set ${data}(color) $command(color)
	}
	if {$command(size) != ""} {set ${data}(symsize) $command(size)}
	if {$command(symbol) != ""} {set ${data}(symbol) $command(symbol)}
    }
    displaylist
}
#--------------------------------------------------------------------
lappend menulist(display) Rescale
lappend menulist(pages) Rescale
set helplist(Rescale) {\
This page is used to modify the stored values in one or 
more buffers. Specify constants to multiply the x and y 
values and constants to add to the x and y values (addition 
is done after multiplication).

It is also possible to transform the x and y scales. The x-axis 
can be transformed between Q, d-space and 2-theta units for an 
arbitrary wavelength.  Intensities (y values) can be transformed 
to square roots [Sqrt(I)], base 10 logarithms [Log(I)]. The 
I/sig(I) option divides each  intensity by its estimated error, 
providing an estimate of the signal-to-noise. Conversion is done 
before values are multiplied or offset.

Changes are applied when the "Rescale" button is pressed.
If more than one scaling or transformation is applied, the 
operation is applied to the results from the previous operation.
If the  "Reset previously applied scaling" checkbox is selected, 
however, the x and y values are reset to the values that were read in,
before any scaling or transformation is done.
}

proc MakeRescale {page}  {
    global command graph
    pack [frame $page.a] -side left -anchor n -fill both
    pack [label $page.a.t1 -text {Select dataset(s) to}] -side top
    pack [label $page.a.t2 -text rescale] -side top
    pack [frame $page.a.2 ] -side top 
    makeselectbox $page.a.2

    pack [frame $page.b] -side left -anchor n -fill both
    pack [frame $page.b.1] -side top -fill both
    pack [frame $page.b.2 -bd 2 -relief groove] -side top -fill both

    set box $page.b.1.a
    pack [frame $box -bd 2 -relief groove] -side left -fill both
    pack [label $box.1 -text "X-axis units"] -side top
    pack [radiobutton $box.2 -text "no change\nor reset" -value {} \
	    -variable command(xmode)] -side top
    pack [radiobutton $box.3 -text d-space -value dspace \
	    -variable command(xmode)] -side top -anchor w
    pack [radiobutton $box.4 -text Q -value Q \
	    -variable command(xmode)] -side top -anchor w
    pack [radiobutton $box.5 -text "2Theta @" -value 2theta \
	    -variable command(xmode)] -side top -anchor w
    pack [frame $box.6] -side top -anchor e 
    pack [label $box.6.a -text A ] -side right
    pack [entry $box.6.b -textvariable command(newwave) -width 8]\
	    -side right

    set box $page.b.1.b
    pack [frame $box -bd 2 -relief groove] -side left -fill both
    pack [label $box.1 -text "Y-axis units"] -side top
    pack [radiobutton $box.2 -text "Linear\n or reset" -value {} \
	    -variable command(ymode)] -side top -anchor w
    pack [radiobutton $box.3 -text Sqrt(I) -value sqrt \
	    -variable command(ymode)] -side top -anchor w
    pack [radiobutton $box.4 -text Log(I) -value log \
	    -variable command(ymode)] -side top -anchor w
    pack [radiobutton $box.5 -text I/sig(I) -value s-n \
	    -variable command(ymode)] -side top -anchor w
    pack [radiobutton $box.6 -text I2/sig(I)2 -value unscale \
	    -variable command(ymode)] -side top -anchor w

    set box $page.b.1.c
    pack [frame $box -bd 2 -relief groove] -side left \
	    -anchor n -fill both
    pack [label $box.t -text Scaling] -side top
    foreach var {xoffset xmult yoffset ymult} \
	    label {"X offset" "X multiplier" \
	    "Y offset" "Y multiplier"} {
	pack [frame $box.$var -bd 2 -relief groove] \
		-side top -fill x
	pack [label $box.$var.l -text $label] -side left
	pack [entry $box.$var.e -textvariable command($var) \
		-width 12] -side right
    }

    set box $page.b.2
    pack [checkbutton $box.3 \
	    -text "Reset previously applied scaling" \
	    -variable command(rescale)] -side top -anchor c
    pack [button $box.4 \
	    -text "Rescale" -command "redefinescale $page.a.2"] \
	    -side top
}

proc PostPageRescale {page} {
    updateselectbox $page.a.2
} 

proc redefinescale {box} {
    global graph command
    foreach num [$box.1 curselection ] {
	set data [$box.1 get $num]
	catch {
	    if $command(rescale) {resetdata $data}
	    ConvertX $data $command(xmode) $command(newwave)
	    ConvertY $data $command(ymode) 
	    scaledata $data \
		    $command(xoffset) $command(xmult) \
		    $command(yoffset) $command(ymult)
	}
    }
    # force an update
    displaylist
}
#--------------------------------------------------------------------
#Combine
lappend menulist(display) Combine
lappend menulist(pages) Combine
set helplist(Combine) {\
The Combine page is used to create a new buffer set by 
adding or subtracting up to five other buffers. The 
buffers must have the same number of data points. No 
attempt is made to offset buffers, so they must all 
start (and end) with the same x value.
}

proc MakeCombine {page}  {
    pack [frame $page.a -bd 4 -relief groove] -side left -fill y -anchor n
    foreach num {1 2 3 4 5} {
	pack [frame $page.a.$num -bd 4 -relief groove] -side top
	pack [label $page.a.$num.1 -text "Select dataset $num"] -side left
	pack [frame $page.a.$num.2] -side left
	pack [label $page.a.$num.3 -text times] -side left
	pack [entry $page.a.$num.4 -textvariable command(combmult$num) -width 5] \
		-side left
    }
    pack [button $page.a.h -text "Sum Files" \
	    -command "CombineFiles"] -side top -anchor center
}

proc PostPageCombine {page} {
    global graph command
    set datalist {(none)}
    foreach data $graph(datalist) {
	global $data
	if {[set ${data}(type)] != "data"} {lappend datalist $data}
    }
    foreach num {1 2 3 4 5} {
	set box $page.a.$num.2.b
	catch {destroy $box}
	eval tk_optionMenu $box command(combdataset$num) $datalist
	pack $box
	set command(combmult$num) 1.0
    }
}

proc CombineFiles {} {
    global command
    set OK 0
    set name {}
    foreach num {1 2 3 4 5} {
	set data $command(combdataset$num)
	if {$data != "(none)" && $command(combmult$num) != 0} {
	    global $data
	    append name "$data*$command(combmult$num)+"
	    incr OK 1
	    if {$OK == 1} {
		set minlength [[set ${data}(xvector)] length]
	    } else {
		set l [[set ${data}(xvector)] length]
		if {$l < $minlength} {set minlength $l}
	    }
	    if {$minlength < 2} {
		tk_dialog .msg "no points" \
		    "There are only $minlength points in file $command(combdataset$num)" "" 0 OK
		return
	    }
	}
    }
    if {$OK == 0} {
	tk_dialog .msg "no files" \
		"No valid files to process" "" 0 OK
	return
    }
    set lastind [expr $minlength - 1]
    set data [string trim $name "+"]
    initdata $data
    global ${data}
    set ${data}(ylabel) "arbitrary"
    set ${data}(xlabel) "2theta"
    set ${data}(xunits) "2theta"
    vector tmpx
    vector tmpy
    vector tmpesd2
    vector sumx
    vector sumy
    vector sumesd2
    set OK 0
    foreach num {1 2 3 4 5} {
	if {$command(combdataset$num) != "(none)" && \
		$command(combmult$num) != 0} {
	    set datn $command(combdataset$num)
	    global $datn 
	    incr OK 1
	    tmpx set [[set ${datn}(xvector)] range 0 $lastind]
	    tmpy set [[set ${datn}(yvector)] range 0 $lastind]
	    tmpy set [tmpy * $command(combmult$num)]
	    tmpesd2 set [[set ${datn}(esdvec)] range 0 $lastind]
	    tmpesd2 set [tmpesd2 * $command(combmult$num)]
	    tmpesd2 set [tmpesd2 * tmpesd2]
	    if {$OK == 1} {
		sumx set tmpx
		sumy set tmpy
		sumesd2 set tmpesd2
	    } else {
		sumx set [sumx + tmpx]
		sumy set [sumy + tmpy]
		sumesd2 set [sumesd2 + tmpesd2]
	    }
	}
    }
    set ${data}(x) [sumx / $OK]
    set ${data}(y) [sumy range 0 end]
    set ${data}(esd) {}
    foreach num [sumesd2 range 0 end] {
	lappend ${data}(esd) [expr sqrt($num)]
    }
    resetdata $data
}

#--------------------------------------------------------------------
lappend menulist(compute) HKLGEN
lappend menulist(pages) HKLGEN
set helplist(HKLGEN) {\
This page generates a list of the allowed reflections for a given 
set of unit cell constants and extinction conditions. It is possible
to specify symmetry by space group or by selecting individual 
translational symmetry operations. 

Space groups should be entered as specified in GSAS:
Enter the centering condition or cell type (P, R, A, B, C, I or F) 
and then the symmetry along each appropriate axis, with spaces between
each axis. Append a final "R" for rhombohedral settings. Some examples 
are: "P 1 21/c 1" or "P 21/c" and "R 3 m" for or "R 3 m R" for the 
hexagonal and rhombohedral settings  of R3m, respectively. If an 
incorrect space group is entered,  no error message is generated, 
but no extinctions are generated. Either direct or reciprocal cell 
parameters may be used.  

The reflections are generated and placed in a separate window when 
the "Compute" button is pressed. This table can be saved to a disk 
file by entering a file name and pressing the "Save as" button. 
}
proc MakeHKLGEN {page}  {
    global graph cellparm extinctlbl
    pack [frame $page.r] -side left -fill both -anchor n
    makecellbox $page.r "Unit Cell\nParameters"
    set cellparm(top) $page.r
    pack [button $page.r.calc -text Compute -command showcell] -side bottom
    makesymbox $page cellparm(laue) setcellbox
    setcellsym $cellparm(laue) setcellbox
    set top $page.m
    pack [frame $top ] -side left -fill both -anchor n
    pack [frame $top.e -relief groove -bd 4] -side top -fill both -anchor n
    set cellparm(extinctmenuH) $top.e 
    label $top.e.l -text "Extinctions to remove"
    grid $top.e.l -in $top.e -row 1 -column 1 -columnspan 2
    label $top.e.el -text "Individual\nconditions"
    grid $top.e.el -in $top.e -row 2 -column 1
    button $top.e.ext -text "0 Set" -command "extinctmenu .extmen"
    grid $top.e.ext -in $top.e -row 2 -column 2
    label $top.e.spgl -text "Space\nGroup:"
    grid  $top.e.spgl -in $top.e -row 3 -column 1
    entry $top.e.spge -textvariable cellparm(spacegroup) -width 8
    grid  $top.e.spge -in $top.e -row 3 -column 2
    grid rowconfigure $top.e 2 -weight 0 -minsize 42
    
    pack [frame $top.lam -relief groove -bd 4] -side top -fill both -anchor center
    pack [label $top.lam.l -anchor e -text "Wavelength:"] \
	    -padx 1m -pady 1m -side left -anchor e
    pack [entry $top.lam.e \
	    -width 8 -relief sunken -bd 2 -textvariable cellparm(wavelength)] \
	    -padx 1m -pady 1m  -side left
    pack [frame $top.ttmax -relief groove -bd 4] -side top -fill both -anchor center
    pack [label $top.ttmax.l -width 10 -anchor e -text "2theta max:"] \
	    -padx 1m -pady 1m -side left
    pack [entry $top.ttmax.e \
	    -width 8 -relief sunken -bd 2 -textvariable cellparm(twothetamax)] \
	    -padx 1m -pady 1m  -side left
}
proc PostPageHKLGEN {page} {
    resetextinctions  
} 
#--------------------------------------------------------------------
lappend menulist(compute) EditCell
lappend menulist(pages) EditCell
set helplist(EditCell) {\
This page generates a stick-figure plot of the allowed reflections 
for a given set of unit cell constants. Unit cells may be input 
with either direct or reciprocal cell parameters. Reflections can 
be omitted or highlighted if extinct due to symmetry. 

Space groups should be entered as specified in GSAS:
Enter the centering condition or cell type (P, R, A, B, C, I or F) 
and then the symmetry along each appropriate axis, with spaces between
each axis. Append a final "R" for rhombohedral settings. Some examples 
are: "P 1 21/c 1" or "P 21/c" and "R 3 m" for or "R 3 m R" for the 
hexagonal and rhombohedral settings  of R3m, respectively. If an 
incorrect space group is entered,  no error message is generated, 
but no extinctions are generated. 

Reflections are generated and displayed when "Compute" is pressed. 
Enter symmetry before pressing "Compute" to omit the extinct 
reflections or afterwards to highlight the reflections. Moving the
sliders or typing a new value causes the reflection positions to be 
recomputed and displayed. 

Individual reflections can be labeled by holding Shift and clicking 
with the left mouse button.  All reflections can be labeled by holding
Shift and double-clicking with the left mouse button and cleared with 
by holding Shift clicking with the right mouse button. Reflection 
labels typically disappear after a period of time (see Options).
Label sizes can similarly be adjusted.
}

proc MakeEditCell {page}  {
    global cellparm
    set cellparm(page) $page
    pack [frame $page.a] -side left -fill y -anchor n
    pack [frame $page.a.3 -bd 4 -relief groove] -side top
    pack [label $page.a.3.1 -text {Select peaklist to edit}] -side top
    pack [frame $page.a.3.peaks] -side top
    pack [label $page.a.1 -text {Select dataset(s) to plot}] -side top
    pack [frame $page.a.2 ] -side top
    pack [listbox $page.a.2.1 -yscrollcommand "$page.a.2.scroll set" \
	    -height 8 -width 15 -bd 2 -relief raised ] -side left
    $page.a.2.1 config -selectmode extended -exportselection 0
    pack [scrollbar $page.a.2.scroll -command "$page.a.2.1 yview"] \
	    -side left -fill y
    bind $page.a.2.1 <Double-Button-1> "getfilelist $page.a.2; SetEditData 1 2 3"
    pack [button $page.a.h -text "PostScript out" -command makepostscriptout] \
	    -side bottom
    pack [button $page.a.p -text "Update Plot" \
	    -command "getfilelist $page.a.2; SetEditData 1 2 3"] -side bottom
    pack [frame $page.b] -side left -fill y -anchor n
    pack [frame $page.b.cell] -side top -fill y
    makesymbox $page.b.cell cellparm(celllaue) SetEditParms

    pack [frame $page.c] -side left -fill y -anchor n
    set top $page.c.ext
    pack [frame $top ] -side top -fill both -anchor n
    set base [frame $page.c.1]
    set cellparm(celleditbase) $base
    pack $base -side top
    set i 0
    foreach var {a b c} {
	incr i
	label $base.l$var -text $var
	grid $base.l$var -in $base -row $i -column 1
	entry $base.e$var -width 7
	grid $base.e$var -in $base -row $i -column 2
	scale $base.s$var -showvalue 0 -orient h  -resolution 0.01
	grid $base.s$var -in $base -row $i -column 3
    }
    foreach var {alpha beta gamma} lbl {a b g} {
	incr i
	label $base.l$var -text $lbl -font symbol
	grid $base.l$var -in $base -row $i -column 1
	entry $base.e$var -width 7
	grid $base.e$var -in $base -row $i -column 2
	scale $base.s$var -showvalue 0 -orient h  -resolution 0.01
	grid $base.s$var -in $base -row $i -column 3
    }
    incr i
    label $base.lzero -text Zero
    grid $base.lzero -in $base -row $i -column 1
    entry $base.ezero -width 7 -textvariable cellparm(zero)
    grid $base.ezero -in $base -row $i -column 2
    scale $base.szero -showvalue 0 -orient h -resolution 0.01 \
	    -command "set cellparm(zero) " -from -0.5 -to 0.5
    grid $base.szero -in $base -row $i -column 3

    pack [frame $top.e -relief groove -bd 4] -side top -fill both -anchor n
    set cellparm(extinctmenuE) $top.e 
    label $top.e.l -text "Extinctions"
    grid $top.e.l -in $top.e -row 1 -column 1
    label $top.e.l1 -text ""
    grid $top.e.l1 -in $top.e -row 1 -column 2
    label $top.e.l2 -text "" -fg #888
    grid $top.e.l2 -in $top.e -row 1 -column 3
    label $top.e.el -text "Individual\nconditions"
    grid $top.e.el -in $top.e -row 2 -column 1
    button $top.e.ext -text "0 Set" -command "extinctmenu .extmen"
    grid $top.e.ext -in $top.e -row 2 -column 2
    label $top.e.extl -text "" -fg #888
    grid $top.e.extl -in $top.e -row 2 -column 3
    label $top.e.spgl -text "Space\nGroup:"
    grid  $top.e.spgl -in $top.e -row 3 -column 1
    entry $top.e.spge -textvariable cellparm(extspg) -width 8
    grid  $top.e.spge -in $top.e -row 3 -column 2
    label $top.e.spg3 -text "" -fg #888
    grid  $top.e.spg3 -in $top.e -row 3 -column 3
    grid rowconfigure $top.e 2 -weight 0 -minsize 42

    pack [frame $page.d] -side left -fill both -anchor center
    pack [frame $page.d.a -bd 4 -relief groove] -side top -fill x -anchor n
    pack [label $page.d.a.1 -justify center -text \
	    "Minimum &\nMaximum Y\nvalues for\ngenerated\npeaks" \
	    ] -side top
    pack [entry $page.d.a.min -width 8 -textvariable cellparm(ymin)] \
	    -side top -anchor center
    pack [entry $page.d.a.max -width 8 -textvariable cellparm(ymax)] \
	    -side top -anchor center
    pack [frame $page.d.w -bd 4 -relief groove] -side top -fill x -anchor n
    pack [radiobutton $page.d.w.u0 -text 2Theta -value 0 \
	     -variable graph(units)] -side top -anchor w
    pack [label $page.d.w.l -text Wavelength] -side top
    pack [entry $page.d.w.e  -width 8] -side top
    pack [radiobutton $page.d.w.u1 -text d-space -value 1 \
	     -variable graph(units)] -side top -anchor w
    pack [radiobutton $page.d.w.u2 -text Q -value 2 \
	     -variable graph(units)] -side top -anchor w
    # disable SetEditData
    set cellparm(skipSetEditData) 1
    trace variable cellparm(editsetting) w SetEditData
}

proc PostPageEditCell {page} {
    global graph cellparm
    catch {destroy $page.a.3.peaks.1}
    set peakslist {}
    foreach data $graph(datalist) {
	global $data
	if {[set ${data}(type)] == "peaks"} {
	    global $data
	    # skip unindexed peak lists
	    if {[llength [set ${data}(h)]] == 0} continue
	    lappend peakslist $data
	}
    }
    lappend peakslist "(new)"
    eval tk_optionMenu $page.a.3.peaks.1 cellparm(editsetting) $peakslist
    pack $page.a.3.peaks.1
    $page.a.3.1 config -text {Select peaklist to edit}
    set base $cellparm(celleditbase)
    set i 0
    foreach var {a b c alpha beta gamma} {
	incr i
	$base.l$var config -fg #888
	$base.e$var config -fg #888 -state disabled
	$base.s$var config -fg #888 -state disabled -command {}
    }
    $page.d.w.l config -fg #888
    $page.d.w.e  config  -fg #888 -state disabled

    updateselectbox $page.a.2
    set cellparm(editsetting) {}
    set cellparm(celleditvar) {}
    setsymbox $page.b.cell {}
    # enable seteditdata
    set cellparm(skipSetEditData) 0
    if {$peakslist == "(new)"} {set cellparm(editsetting) "(new)"}
}

# Called to select peaklist and set it up for plotting
proc SetEditData {a1 a2 a3} {
    global cellparm graph
    set page $cellparm(page)
    if {$cellparm(editsetting) == ""} return
    if $cellparm(skipSetEditData) return
    #------------------------------------------------------------
    # allow a new dgen to be created
    if {$cellparm(editsetting) == "(new)"} {
	editnew
	return
    }
    #------------------------------------------------------------
    set cellparm(extspg) (none)
    set cellparm(celleditvar) $cellparm(editsetting)
    editold
    set data $cellparm(celleditvar)
    global $data
    set base $cellparm(celleditbase) 
    set i 0
    foreach var {a b c alpha beta gamma} {
	incr i
	$base.l$var config -fg black
	$base.e$var config -textvariable ${data}($var) \
		-fg black -state normal
	set value [set ${data}($var)]
	catch {
	    $base.s$var config -fg black -state normal \
		    -from [expr $value * (1-$cellparm(sens)/100.)]  \
		    -to [expr $value * (1+$cellparm(sens)/100.)]
	    $base.s$var set $value
	    $base.s$var config -command "set ${data}($var) "
	}
	# -variable ${data}($var)
    }
    $page.d.w.l config -fg black
    $page.d.w.e config -textvariable ${data}(wavelength) -fg black \
	    -state normal
    set cellparm(celllaue) [set ${data}(laue)]

    # recompute the reflections using the current settings
    if {[validatecell] == ""} computehkllist
    # trigger a recalc of the extinctions
    set cellparm(oldextspg) {}
    $cellparm(extinctmenuE).spg3 config -text "[set ${data}(spg)]"
    set count 0
    foreach num [set ${data}(extcodes)] {
	if {$num > 0} {incr count}
    }
    $cellparm(extinctmenuE).extl config -text "$count Set"
    # get the plot list and add the peak file
    getfilelist $page.a.2
    # if the data array appears in the plot list remove it
    set num [lsearch -exact $graph(plotlist) $data]
    if {$num != -1} {
	set graph(plotlist) [lreplace $graph(plotlist) $num $num]
	# lappend graph(plotlist) $data
    }
    eval $graph(blt) element delete [$graph(blt) element names]
    eval $graph(blt) marker delete  [$graph(blt) marker names]
    foreach pldat $graph(plotlist) {
	global $pldat ${pldat}(xvector) ${pldat}(yvector)
	# expression to select each color from the list 
	$graph(blt) element create $pldat \
		-linewidth [set ${pldat}(line)] \
		-color [set ${pldat}(color)] -symbol [set ${pldat}(symbol)] \
		-xdata [set ${pldat}(xvector)] -ydata [set ${pldat}(yvector)] \
		-pixels [expr 0.125*[set ${pldat}(symsize)]]i
#		-activecolor black 
	[set ${pldat}(xvector)] notify now
	[set ${pldat}(yvector)] notify now
    }
    $graph(blt) element create extinct \
	    -color yellow -linewidth 2 \
	    -xdata xextinct -ydata yextinct -symbol none
#	    -activecolor red
    $graph(blt) element create $data \
	    -color blue -linewidth [set ${data}(line)]  -symbol none \
	    -xdata [set ${data}(xvector)] -ydata [set ${data}(yvector)]
#	    -activecolor red
    xextinct notify now
    yextinct notify now
    [set ${data}(xvector)] notify now
    [set ${data}(yvector)] notify now
    setsymbox $page.b.cell $cellparm(celllaue)
    setcellsym $cellparm(celllaue) SetEditParms
    RecalcLoop
    bind $graph(blt) <Shift-Button-1> "lblhkl %W %x"
    bind $graph(blt) <Shift-Double-Button-1> "lblallhkl %W"
    bind $graph(blt) <Shift-Button-3> "delallhkllbl %W"
}

# this is called when the laue symmetry is changed or by setcellsym
proc SetEditParms {fix lock} {
    global cellparm
    if {$cellparm(editsetting) == ""} return
    set data $cellparm(celleditvar)
    global $data
    set base $cellparm(celleditbase)
    
    set i -1
    foreach var {a b c alpha beta gamma} {
	incr i
	# variables to fix
	if {[lindex $fix $i]-0 > 0} {
	    $base.l$var config -fg #808080
	    $base.e$var config -state disabled -fg #808080 \
		    -textvariable ${data}($var)
	    $base.s$var config -state disabled -fg #808080 \
		    -command {}
	    set ${data}($var)  [lindex $fix $i]
	} else {
	    $base.l$var config -fg black
	    $base.e$var config -state normal -fg black \
		    -textvariable ${data}($var)
	    set value [set ${data}($var)]
	    catch {
		$base.s$var config -state disabled -fg black \
			-from [expr $value * (1-$cellparm(sens)/100.)]  \
			-to [expr $value * (1+$cellparm(sens)/100.)]
		$base.s$var set $value
		$base.s$var config -command "set ${data}($var)"
	    }
	    if {$cellparm(editsetting) != "(new)"} {
		$base.s$var config -state normal
	    }
	}
	# variables to lock to others
	if {"[lindex $lock $i]" != ""} {
	    $base.l$var config -fg #888 
	    $base.e$var config -state disabled -fg #888 \
		    -textvariable ${data}([lindex $lock $i])
	    $base.s$var config -state disabled -fg #888 -command {}
	}
    }
}

#--------------------------------------------------------------------
lappend menulist(compute) Fit
lappend menulist(pages) Fit
set helplist(Fit) {\
Perform a profile fit on a region of a dataset using a  pseudo-Voigt
and optionally Finger-Cox-Jephcoat low-angle asymmetry.

Steps:
  1) Select a dataset and peaklist (when needed)
  2) Define a fit region with the zoom and "Set Range to fit" 
       Shortcuts: use the S/Z/F/B/E keys (see shortcuts)
  3) Add peaks (when needed) using Control-double click at the peak
       or press a "Set" button and then use Control-click at the peak
  4) Select parameters to refine
        It is best to first refine the width and areas and later refine
        background, positions and eta
  5) Refine by pressing the "Run GPLSFT" button

After a refinement has been the refinement can be reversed once with 
the "Undo" button or the fit curve can be saved with "Store Fit".

Low angle asymmetry can be modeled using the Finger-Cox-Jephcoat 
correction. This requires the instrument diameter and the sample 
and detector heights, all in the same units.

Fit results can be superimposed on the plot or shown in a separate 
window; see "Report fit results" in the the Options command.
}

proc MakeFit {page} {
    global graph gpls
    pack [frame $page.a] -side left -fill y -anchor n
    pack [frame $page.a.d -bd 4 -relief groove] -side top
    pack [label $page.a.d.1 -text {Select a dataset to fit}] -side top
    pack [frame $page.a.p -bd 4 -relief groove] -side top -fill both
    pack [label $page.a.p.1 -text {Select a peak list}] -side top -fill both
    pack [frame $page.a.3 -bd 4 -relief groove] -side top -fill both
    pack [button $page.a.3.a -text {Set range to fit} \
	    -command SetFitRange \
	    ] -side top -anchor center
    pack [label  $page.a.3.b -textvariable gpls(rangemsg)] -side top -anchor center
    pack [frame $page.a.4 ] -side top
    pack [label $page.a.4.a -text {Max cycles}] -side left
    pack [entry $page.a.4.b -justify center \
	    -textvariable gpls(ncyc) -width 4] -side left
    pack [frame $page.a.5 ] -side top
    pack [label $page.a.5.a -text {Damping factor}] -side left
    pack [entry $page.a.5.b -justify center \
	    -textvariable gpls(damp) -width 4] -side left
    button $page.a.6 -command wrgpls -text {Run GPLSFT} -width 12
    pack [frame $page.a.8 -bg red -bd 4] -side top -fill x
    pack [label  $page.a.8.a -text "Unable to refine:" -fg white -bg red] -side top -anchor w
    pack [label  $page.a.8.b -textvariable graph(gplsstatus)]
    set graph(gplsstatus) {}
    pack [frame $page.b -bd 4 -relief groove] -side left -fill y -anchor n
    frame $page.b.plot
    pack [button $page.b.plot.a -text "PostScript out" \
	    -command makepostscriptout] -side left
    pack [button $page.b.plot.b -text "Store Fit" -command StoreFit] -side right	    
    button $page.b.undo -text "Undo last cycle" -command UndoFit
    set gpls(overallframe) $page.b 
    overalltable
    pack [frame $page.c -bd 4 -relief groove] -side left -fill both -anchor n
    set gpls(peakframe) $page.c
    #peaktable

    if {[trace vinfo  gpls(dataset)] == ""} {
	trace variable gpls(dataset) w setupfitfile
    }
    if {[trace vinfo  gpls(peaklist)] == ""} {
	trace variable gpls(peaklist) w setupfitfile
    }
    if {[trace vinfo  gpls(asymflg)] == ""} {
	trace variable gpls(asymflg) w disableasym
    }
    set gpls(page) $page
    if {[trace vinfo  gpls] == ""} {
	trace variable gpls w enablegpls
    }
}

proc PostPageFit {page}  {
    global graph gpls
    set dataonlylist {}
    set peakonlylist {}
    foreach data $graph(datalist) {
	global $data
	if {[set ${data}(type)] != "peaks"} {lappend dataonlylist $data}
	if {[set ${data}(type)] == "peaks"} {lappend peakonlylist $data}
    }
    catch {pack forget $page.a.7}

    catch {destroy $page.a.d.a}
    catch {destroy $page.a.p.a}
    if {$dataonlylist != ""} {
	eval tk_optionMenu $page.a.d.a gpls(dataset) $dataonlylist
	pack $page.a.d.a
	peaktable
    } else {
	pack [label $page.a.d.a -text {no data to fit} -fg red]
	return
    }
    # if there are no peak lists, create one
#    if {$peakonlylist == ""} {
#	initpeaks peaklist1
#	set peakonlylist peaklist1
#    }
    lappend peakonlylist {(create new)}
    eval tk_optionMenu $page.a.p.a gpls(peaklist) $peakonlylist
    pack $page.a.p.a

    set gpls(asymflg) $gpls(asymflg)
    foreach i {1 2 3 4} {set gpls(refine_$i) 0}
    bind $graph(blt) <Control-Double-Button-1> "addpeak %W %x %y"
    bind $graph(blt) <Control-Button-1> " "
    # if there is only 1 fit to fit choose it
    if {[llength $dataonlylist] == 1} {
	set gpls(dataset) $dataonlylist
    } else {
	$graph(blt) config -title "Select a dataset to fit"
	set gpls(dataset) "(none selected)"
	set graph(plotlist) {}
	displaylist
    }
    set gpls(rangemsg) {No range set}
}

#--------------------------------------------------------------------
# Smoothing and Peak finding
# GUI section by A.McGhie don't blame B.Toby
#      reorganized 1/21/00 -- do blame B.Toby
# Fortran program by R.Harlow
# mcghieaj@esvax.email.dupont.com
# 01/20/2000
#

# initial conditions
# gaussian(ndp)  = number of data point for gausian to fit
# gpls(fwhm) = full width half max (defaults in fit)
# gaussian(sens) = sensitivity
set gaussian(esd)  2.0000
set gaussian(sens) 3.0000
set gaussian(dp) 1
set gaussian(program) [file join $scriptdir gaussuper]


lappend menulist(compute) "PeaksSmooth PeakSearch"
lappend menulist(compute) "PeaksSmooth Smooth"
lappend menulist(pages) PeaksSmooth
set helplist(PeaksSmooth) {\
Used to smooth diffraction patterns with a Gaussian. 
The smoothed results are used to compute a first and 
second derivative of the data. Peak positions, as well 
as height and area estimates are made using the second 
derivative. From the peak positions, data away from the 
peak positions are used to estimate the background. 

FWHM sets the width of the Gaussian convolution function. 
It appears best to set this to the FWHM of the actual 
peaks.

The "Sensitivity" and "ESD" settings are used only for 
peak searching. Both are used to reject peaks below 
signal-to-noise criteria -- somehow.
}

proc MakePeaksSmooth {page} {
    global graph gaussian
    pack [frame $page.a] -side left -fill y -anchor n
    pack [label $page.a.t1 -text {Select dataset(s) }] -side top
    pack [frame $page.a.2 ] -side top 
    makeselectbox $page.a.2
    pack [frame $page.a1 -borderwidth 3 -relief groove]  -side left -fill y -anchor n
#    pack [frame $page.a1.b] -side left -fill y -anchor n

    grid [label $page.a1.l2 -text {FWHM } ] -column 1 -row 1 -columnspan 2 -sticky e
    grid [entry $page.a1.e2 -justify center \
	    -textvariable gpls(fwhm) -width 8] -column 3 -row 1
    grid [label $page.a1.j2  -justify left \
	    -text "defines the gaussian width for convolution.\nShould be about the same as the peak width." \
	    ] -column 4 -row 1 -sticky w

    grid [label $page.a1.l3 -text {Sensitivity } ] -column 1 -row 2 -columnspan 2 -sticky e
    grid [entry $page.a1.e3 -justify center \
	    -textvariable gaussian(sens) -width 8] -column 3 -row 2
    grid [label $page.a1.j3  -justify left \
	    -text "relative error threshold for peak location." \
	    ] -column 4 -row 2 -sticky w

    grid [label $page.a1.l1 -text {ESD } ] -column 1 -row 3 -columnspan 2 -sticky e 
    grid [entry $page.a1.e1 -justify center \
	    -textvariable gaussian(esd) -width 8] -column 3 -row 3
    grid [label $page.a1.j1  -justify left \
	    -text "reject peaks below this signal-to-noise ratio." \
	    ] -column 4 -row 3 -sticky w

    # expand the empty row, pushing the above to the top
    grid rowconfigure $page.a1 4 -weight 2
    grid [label $page.a1.ll -text Save:] -row 5 -column 1 -sticky w
    grid [checkbutton $page.a1.do -text "Smoothed curve" \
	    -variable gaussian(d0) ] -row 6 -column 2 -columnspan 2 -sticky w
    grid [checkbutton $page.a1.db -text "Background" \
	    -command "disablePeakVariables $page" \
	    -variable gaussian(db) ] -row 7 -column 2 -columnspan 2 -sticky w
    grid [checkbutton $page.a1.d1 -text "1st Derivative" \
	    -variable gaussian(d1) ] -row 8 -column 2 -columnspan 2 -sticky w
    grid [checkbutton $page.a1.d2 -text "2nd Derivative" \
	    -variable gaussian(d2) ] -row 9 -column 2 -columnspan 2 -sticky w
    grid [checkbutton $page.a1.dp -text "Peak positions" \
	    -command "disablePeakVariables $page" \
	    -variable gaussian(dp) ] -row 10 -column 2 -columnspan 2 -sticky w
    # stick in a bit more space
    grid rowconfigure $page.a1 11 -weight 1
    grid [button $page.a1.b1 -text Compute \
		-command "GaussianMyFile $page.a.2" ] -row 12 -column 1 -columnspan 3
    pack [button $page.a.4    -text "  Update Plot  " \
	    	-command "getfilelist $page.a.2; displaylist"] -side bottom
}

proc PostPagePeaksSmooth {page}  {
    global graph gpls
    updateselectbox $page.a.2
    disablePeakVariables $page
}

proc disablePeakVariables {page} {
    global gaussian
    if {$gaussian(dp) || $gaussian(db) } {
	foreach i {l1 j1 l3 j3} {
	    $page.a1.$i config -fg black
	}
	$page.a1.e3 config -state normal -fg black
	$page.a1.e1 config -state normal -fg black
    } else {
	foreach i {l1 j1 l3 j3} {
	    $page.a1.$i config -fg gray
	}
	$page.a1.e3 config -state disabled -fg gray
	$page.a1.e1 config -state disabled -fg gray
    }
}

proc GaussianMyFile {box} {
    global graph gaussian gpls command
    set opt 0
    foreach item {d0 db d1 d2 dp} {
	set opt [expr $opt || $gaussian($item)]
    }
    if !{$opt} {
	tk_dialog .msg "nothing to do" \
		"Nothing to do: you have not selected any output options" \
		error 0 OK
	return
    }

    set type 3
    if {$gaussian(dp) || $gaussian(db)} {set type 4}

    set data "(none)"
    # now loop over selected data sets
    foreach num [$box.1 curselection] {
	set data  [$box.1 get $num]
	global ${data}
	if {[set ${data}(type)] == "xy"} {

	    set fp [open gauss.inp w]
	    set number_of_data_points [[set ${data}(xvector)] length]
	    set max_two_theta  [[set ${data}(xvector)] range end end]
	    set min_two_theta  [[set ${data}(xvector)] range 0 0]
	    set delta [expr ($max_two_theta - $min_two_theta) \
		    / ($number_of_data_points - 1)]
	    puts $fp [format "%1d %d %f %f %f %f %f %f " \
		    $type $number_of_data_points \
		    $min_two_theta $max_two_theta $delta \
		    $gpls(fwhm) $gaussian(sens) $gaussian(esd)]
	    puts $fp [[set ${data}(xvector)] range 0 end]
	    puts $fp [[set ${data}(yvector)] range 0 end]
	    puts $fp [[set ${data}(esdvec)] range 0 end]
	    close $fp

	    #    run Harlow's fortran program
	    exec $gaussian(program) < gauss.inp > gauss.out

	    set opt 0
	    # get the x-axis data when needed
	    foreach item {d0 db d1 d2} {
		if {$gaussian($item)} {
		    source gauss.x
		    break
		}
	    }
	    foreach label {"smoothed intensity values" background \
		    "1st derivative" "2nd derivative"} \
		    lbl {smo bkg 1st 2nd} \
		    item {d0 db d1 d2} {
		if {$gaussian($item)} {
		    # create a name
		    set newdata ${data}_[expr round( $gpls(fwhm) * 1000)]
		    if {$lbl == "bkg"} {
			append newdata _[expr round($gaussian(sens) * 10)]
			append newdata _[expr round($gaussian(esd) * 10)]
		    }
		    append newdata _$lbl
		    source gauss.$lbl
		    
		    initdata $newdata
		    global ${newdata}
		    set ${newdata}(xlabel) "2theta"
		    set ${newdata}(xunits) "2theta"
		    set ${newdata}(ylabel) $label
		    set ${newdata}(skip) 0
		    set ${newdata}(x) $newx
		    set ${newdata}(y) $newy
		    set ${newdata}(esd) $newesd
		    resetdata $newdata
		    # add the new file to the plot list
		    if {[lsearch $graph(plotlist) $newdata] == -1} {
			lappend graph(plotlist) $newdata
		    }

		}
	    }
	    # now process peaks
	    if {$gaussian(dp)} {
		source gauss.pks
		set peakdata ${data}
		append peakdata _[expr round($gpls(fwhm) * 1000)]
		append peakdata _[expr round($gaussian(sens) * 10)]
		append peakdata _[expr round($gaussian(esd) * 10)]_pks
		initpeaks $peakdata
		global $peakdata
		set ${peakdata}(xlabel) "2theta"
		set ${peakdata}(xunits) "2theta"
		set ${peakdata}(ylabel) [set ${data}(ylabel)]
		set ${peakdata}(x) $tt
		set ${peakdata}(y) $area
		set ${peakdata}(widths) $wid
		set ${peakdata}(heights) $height
		set ${peakdata}(bkgs) $bkg
		resetdata $peakdata
		# add the new file to the plot list
		if {[lsearch $graph(plotlist) $peakdata] == -1} {
		    lappend graph(plotlist) $peakdata
		}
	    }
	}
    }
    if {$data == "(none)"} {
	tk_dialog .msg "nothing to do" \
		"Error: you have not selected any files to process" \
		error 0 OK
	return
    }
    updateselectbox $box
    displaylist
    # cleanup files
    if {!$command(debug)} {
	foreach lbl {smo bkg 1st 2nd inp out x pks} {
	    file delete gauss.$lbl
	}
    }
}
#--------------------------------------------------------------------
lappend menulist(compute) Index
lappend menulist(pages) Index
set helplist(Index) {
An interface to autoindexing routines TREOR, ITO and DICVOL

Note that peaks must be in units of 2theta, Q or d-space.
}
# see index.tcl for routines
#--------------------------------------------------------------------
#lappend menulist(options) Options
#lappend menulist(pages) Options
set helplist(Options) {\
Used to set values used in other parts of the program.
  "Plot legend" causes a legend to be shown on the side of the plot.
  "Write PostScript" and "Print PostScript" (UNIX only) determine
      how hardcopy is generated.

  "Report fit results in plot window" places fit output in plot
     or a separate window.

  "2theta max" determines the range of reflections generated
  "max change" controls the slider range and sensitivity
  "HKL label erase time" specifies the time to display reflection
     labels. 0 means delete them only with the right-mouse button
  "HKL label size" specifies the reflection labels size in pixels
     (or points for negative values).

  "Help Options" determine if help is displayed in a Web browser
     or a separate window.
}

proc MakeOptions {page}  {
    global command graph tcl_version
    #global tcl_platform
    catch {toplevel $page}
    eval destroy [pack slaves $page]
    raise $page
    set command(options_page) $page
    pack [frame $page.1] -side left -anchor n
    pack [frame $page.1.a -bd 4 -relief groove] -side top -anchor w
    pack [label $page.1.a.1 -text {Select plot options}] -side top
    pack [checkbutton $page.1.a.2 \
	    -text "Plot Legend" -variable graph(legend) \
	    -command graphlegend] -side top -anchor w
    pack [checkbutton $page.1.a.3 -text "Write PostScript files" \
	    -variable graph(printout) -offvalue 0 -onvalue 1 \
	    -command "setprintopt $page.1.a"] -side top -anchor w
    pack [frame $page.1.a.4] -side top -anchor w -fill x
    pack [entry $page.1.a.4.2 -textvariable graph(outname)] -side right
    pack [label $page.1.a.4.1 -text "PostScript file name:"] -side left
    pack [checkbutton $page.1.a.5 -text "Print PostScript files" \
	    -variable graph(printout) -offvalue 1 -onvalue 0 \
	    -command "setprintopt $page.1.a" ] -side top -anchor w
    pack [frame $page.1.a.6] -side top -anchor w -fill x
    pack [entry $page.1.a.6.2 -textvariable graph(outcmd)] -side right 
    pack [label $page.1.a.6.1 -text "Command to print files:"] -side left

    pack [frame $page.1.b -bd 4 -relief groove] -side top -fill x -anchor w
    pack [label $page.1.b.1 -text {Peak fit options}] -side top
    pack [checkbutton $page.1.b.2 -text "Report fit results in plot window" \
	    -variable gpls(fitonplot)] -side top -anchor w

    pack [frame $page.2] -side left -anchor n
    pack [frame $page.2.c -bd 4 -relief groove] -side top  -anchor w
    pack [label $page.2.c.1 -text {EditCell options}] -side top
    pack [frame $page.2.c.y] -side top -fill both
    pack [frame $page.2.c.a] -side top  -anchor w
    pack [label $page.2.c.a.l -width 10 -anchor e -text "2theta max:"] \
	    -padx 1m -pady 1m -side left
    pack [entry $page.2.c.a.e \
	    -width 6 -bd 2 -textvariable cellparm(twothetamax)] \
	    -padx 1m -pady 1m  -side left
    pack [frame $page.2.c.b] -side top  -anchor w
    pack [label $page.2.c.b.l -text "Max change\nin cell:"] -side left
    pack [entry $page.2.c.b.e -textvariable cellparm(sens) -width 3] \
	    -side left
    pack [label $page.2.c.b.l1 -text %] -side left
    pack [frame $page.2.c.c] -side top  -anchor w
    pack [label $page.2.c.c.l -text "HKL label\nerase time:"] -side left
    pack [entry $page.2.c.c.e -textvariable cellparm(fadetime) -width 5] \
	    -side left
    pack [label $page.2.c.c.l1 -text seconds] -side left
    pack [frame $page.2.c.d] -side top  -anchor w
    pack [label $page.2.c.d.l -text "HKL label size:"] -side left
    pack [entry $page.2.c.d.e -textvariable cellparm(fontsize) -width 3] \
	    -side left
    pack [label $page.2.c.d.l1 -text pixels] -side left
    # old versions if tcl/tk don't support the font command
    if {$tcl_version < 8.0} {
	$page.2.c.d.l config -fg #888
    	$page.2.c.d.e config -fg #888 -state disabled
	$page.2.c.d.l1 config -fg #888
    }
#    if {$tcl_platform(platform) == "unix"} {
	pack [frame $page.2.h -bd 4 -relief groove] -side top -fill x -anchor w
	pack [label $page.2.h.1 -text {Help options}] -side top
	pack [radiobutton $page.2.h.2 \
		-text "Help in browser" -variable command(helpmode) \
		-value 0] -side top -anchor w
 	pack [radiobutton $page.2.h.3 \
		-text "Help in help window" -variable command(helpmode) \
		-value 1] -side top -anchor w
#   }

    pack [button $page.1.q -text Close -command "destroy $page"] \
	    -side top 
   PostPageOptions $page
   bind $page <Key-F1> {MakeHelp .help options}
}

proc setprintopt {page} {
    global graph tcl_platform
    if {$tcl_platform(platform) == "windows"} {
	set graph(printout) 1
	$page.3 config -state disabled -fg black
	$page.5 config -fg #888 -state disabled
    }
    if $graph(printout) { 
	$page.4.1 config -fg black
	$page.4.2 config -fg black -state normal
	$page.6.1 config -fg #888 
	$page.6.2 config -fg #888 -state disabled
    } else {
	$page.4.1 config -fg #888 
	$page.4.2 config -fg #888 -state disabled
	$page.6.1 config -fg black
	$page.6.2 config -fg black -state normal
    }
}

proc PostPageOptions {page} {
    setprintopt $page.1.a
    graphlegend
}

#--------------------------------------------------------------------
#--------------------------------------------------------------------
#lappend menulist(?) X..X
#lappend menulist(pages) X..X
#set helplist(X..X) {}
#proc MakeX..X {page}  {}
#proc PostPageX..X {page} {} 
#--------------------------------------------------------------------
# show the 'about' message
proc aboutcmpr {} {
    global Revision
    tk_dialog .warn About "
Program CMPR\n\n\
B. Toby, NIST\nBrian.Toby@NIST.gov\n\
Not subject to copyright\n\n\
Included codes:\n\
* HKLGEN: C. Hubbard, J. Stalick, A. Mighell & others\n\
* GPLSFT: D. Cox, W. Hamilton, L. Finger & many others\n\
* Space group extinctions: A. Larson & NRC Canada\n\
* Gaussian smoothing/peak search: R. Harlow & A. McGhie\n\
* ITO: J. W. Visser\n\
* DICVOL: A. Boultif and D. Lour\n\
* TREOR: P.-E. Werner\n\
$Revision"  {} 0 OK
}
#--------------------------------------------------------------------
#--------------------------------------------------------------------
#lappend menulist(display) Help
#lappend menulist(pages) Help
proc MakeHelp {page "opt {}"}  {
    global helplist helpmsg graph command tcl_platform env scriptdir
    if {$opt == ""} {set opt $command(pagenow)}
    set helpmsg {}
    if {!$command(helpmode)} {
	set url [file join $scriptdir doc cmpr.html]
	# urls need to be absolute, at least for -remote in UNIX
	if {[string index $url 0] == "."} {
	    set url [file join [pwd] $url]
	    regsub "/\./" $url "/" url
	}
	set link $opt
	if {$tcl_platform(platform) == "windows"} {
	    if {[file exists $env(COMSPEC)]} {
		pleasewait "starting web browser"
		after 3000 donewait
		exec $env(COMSPEC) /c start $url &
		return
	    }
	} else {
	    # try to use a running netscape, if possible
	    if [catch {
		exec netscape -remote openURL(${url}#$link)
	    }] {
		pleasewait "starting Netscape"
		after 2000 donewait
		exec netscape ${url}#$link & 
	    }
	    return
	}
    }

    catch {toplevel $page}
    eval destroy [grid slaves $page]
    raise $page
    grid [label $page.0 -text \
	    "Click on a topic for information" ] \
	    -column 0 -row 0 -columnspan 3
    listbox $page.cmds -relief raised -bd 2 -yscrollcommand \
	    "$page.scroll set" -height 10 -width 0
    scrollbar $page.scroll -command "$page.cmds yview"
    bind $page.cmds <ButtonRelease-1> \
	    "+set helpmsg \
	    \$helplist(\[$page.cmds get \
	    \[$page.cmds curselection\]\])"
    grid $page.scroll -column 0 -row 1 -sticky ns
    grid $page.cmds  -column 1 -row 1 -sticky news
    grid [message $page.help -textvariable helpmsg \
	     -width 70c -relief groove] \
	   -column 2 -row 1 -rowspan 2  -sticky nsew
    grid [button $page.q -text Close -command "destroy $page"] \
	    -column 0 -row 2 -columnspan 2
    PostPageHelp $page $opt
    grid columnconfigure $page 2 -weight 1
    grid rowconfigure $page 2 -weight 1
}
proc PostPageHelp {page "opt {}"} {
    global helplist command helpmsg
    $page.cmds delete 0 end
    set nameslist [lsort [array names helplist]]
    eval $page.cmds insert end $nameslist
    if {[$page.cmds curselection] == ""} {$page.cmds selection set 0}
    # see if we can find help for the current page
    set num [lsearch [string tolower $nameslist] $opt]
    if {$num == -1} return
    $page.cmds selection clear 0 end
    $page.cmds see $num
    $page.cmds selection set $num
    set helpmsg $helplist([$page.cmds get $num])
}
bind . <Key-F1> {MakeHelp .help}
#--------------------------------------------------------------------
#--------------------------------------------------------------------
proc graphlegend {} {
    global graph blt_version
    if {$blt_version >= 2.3 && $blt_version < 8.0} {
	if $graph(legend) {
	    $graph(blt) legend config -hide no
	} else {
	    $graph(blt) legend config -hide yes
	}
    } else {
	if $graph(legend) {
	    $graph(blt) legend config -mapped yes
	} else {
	    $graph(blt) legend config -mapped no
	}
    }
}

proc makepostscriptout {} {
    global graph
    if !$graph(printout) {
	set out [open "| $graph(outcmd) >& plot.msg" w]
	catch {
	    puts $out [$graph(blt) postscript output -landscape 1 \
		-decorations no -height 7.i -width 9.5i]
	    close $out
	} msg
	catch {
	    set out [open plot.msg r]
	    if {$msg != ""} {append msg "\n"}
	    append msg [read $out]
	    close $out
	    file delete plot.msg
	}
	if {$msg != ""} {
	    tk_dialog .msg "file created" \
		    "Postscript file processed with command \
		    $graph(outcmd). Result: $msg" "" 0 OK
	} else {
	    tk_dialog .msg "file created" \
		    "Postscript file processed with command \
		    $graph(outcmd)" "" 0 OK
	}
    } else {
	$graph(blt) postscript output $graph(outname) -landscape 1 \
		-decorations no -height 7.i -width 9.5i    
	tk_dialog .msg "file created" \
		"Postscript file $graph(outname) created" "" 0 OK
    }
}

#-------------------------------------------------------------------------
#process command line arguments
set cnt 0
set format {}
while {$cnt < [llength $argv]} {
    set fmt [lindex $argv $cnt]
    if {[string range $fmt 0 0] == "-"} {
	incr cnt
	set fmtnum [lsearch $command(cmdopt) $fmt]
	if {$fmtnum < 0} {
	    puts "Unknown option $fmt, Known formats are:"
	    puts $command(cmdopt)
#	    destroy .
	    exit
	}
	set format $fmt
    }
    if {$format == ""} {
	puts "No format specified, Known formats are:"
	puts $command(cmdopt)
#	destroy .
	exit
    }
    foreach file [lindex $argv $cnt] {
	puts "Reading $file with $command($format)"
	if [file exists $file] {
	    set ret [$command($format) $file]
	    if {$ret != ""} {
		tk_dialog .err "Read Error" \
			"Error reading $format file\n$file:\n$ret" \
			error 0 OK
	    }
	} else {
	    puts "File $file not found"
	}
    }
    incr cnt
}
#-------------------------------------------------------------------------
# set up zooming of the axes; modify zooming so that it feeds back
# to the fit page
if [catch {
    Blt_ZoomStack $graph(blt)
    Blt_ActiveLegend $graph(blt)
} errmsg] {
    tk_dialog .err "BLT Error" \
	    "Error in BLT setup, support routine problem: $errmsg \
	    The pkgIndex.tcl is probably not loading bltGraph.tcl" \
	    error 0 "Limp ahead"
}
$graph(blt) xaxis configure -title {}
$graph(blt) yaxis configure -title {}
$graph(blt) config -title {} -plotbackground white
$graph(blt) legend config -font  *-Helvetica-Bold-R-Normal-*-10-*-*
#-------------------------------------------------------------------------
# define a menubar
catch {destroy .a}
grid [frame .a -bd 4 -relief groove -class MenuFrame] \
	-column 0 -row 0 -sticky nsew

foreach menu {File Display Compute Options} {
    set m [string tolower $menu]
    pack [menubutton .a.$m -text $menu -underline 0 -menu .a.$m.menu] \
	    -side left
    menu .a.$m.menu
    foreach item $menulist($m) {
	set cmd [lindex $item 0]
	set lbl [lindex $item 1]
	if {$lbl == ""} {set lbl $cmd}
	.a.$m.menu add command \
		-label $lbl -command "DoCommand $cmd"
	if {$command(debug) && [catch {set helplist($cmd)}]} {puts "no help for $cmd"}
    }
}
# special menus
.a.options.menu add command -command "MakeOptions .options" -label Options

pack [menubutton .a.help -text Help -underline 0 -menu .a.help.menu] -side right
menu .a.help.menu -tearoff 0
.a.help.menu add command -command "MakeHelp .help" -label "Help on current page"
.a.help.menu add command -command aboutcmpr -label "About"

#-------------------------------------------------------------------------
# create a frame for the notebook page
set command(notebook) .b
if $command(haveBW) {
    NoteBook $command(notebook) -bd 2
    # create a notebook page for each dialog box
    foreach page $menulist(pages) {
	set pagename [string tolower $page]
	set notepage [$command(notebook) insert end $pagename -text $page ]
	# create the largest page
	if {$page == "EditCell" || $page == "Index"} {
	    Make$page $notepage
	} else {
	    $command(notebook) itemconfigure $pagename \
		-createcmd "Make$page $notepage" 
	}
	$command(notebook) itemconfigure $pagename -raisecmd \
		"set command(pagenow) [string tolower $pagename]; \
		PostPage$page $notepage; \
		$command(notebook) compute_size"
    }
} else {
    Notebook:create $command(notebook) -pages $menulist(pages)
    # fill the notebook page for each dialog box
    foreach page $menulist(pages) {
	set pagename [Notebook:frame $command(notebook) $page]
	Make$page $pagename
	Notebook:pageconfig $command(notebook) $page -command \
		"set command(pagenow) [string tolower $page]; PostPage$page $pagename"
    }
}

grid $command(notebook) -column 0 -row 1 -sticky nsew
grid columnconfig . 0 -weight 1
grid rowconfig . 1 -weight 1
graphlegend
# set the page to the first command
DoCommand [lindex $menulist(pages) 0]
update
ResizeNotebook
