#-----------------------------------------------------------------
# read BT1 multidetector data directly from the instrument
#-----------------------------------------------------------------
# This will run only in UNIX and only if fetchBT1 has been installed on 
# the local computer
# This will probably work outside NIST, if you really want to 
# see data as it is collected at BT1, in which case you know who 
# to contact to get a copy of fetchBT1
#------------------------------------------------------------------

# test if the command to get the file exists on this computer
if ![catch {exec which fetchBT1} ] {

    #--------define a dialog box entry
    lappend command(readbuttons) "Read Active file on BT-1"
    lappend command(readbuttonproc) ReadBT1active

    proc ReadBT1active {} {
	global command
	pleasewait "reading the active file"
	set ret [readBT1active]
	donewait
	if {$ret != ""} {return $ret}
	showlastentry $command(read_filelist)
    }

    #--------------------------------------------------------------------------
    # load multi detector data directly from BT1
    proc readBT1active {"zerocorr 1" "multcorr 1"} {
	#   zerocorr != 0 ==> apply detector zero correction
	#   multcorr != 0 ==> apply detector sensitivity correction
	#
	global bt1 graph
	
	set ndet 32
	set lnum 0
	set scan 1
	# signal errors by quitting
	if [catch {
	    set fl [open "| fetchBT1" r]
	    while {[gets $fl line] >= 0} {
		incr lnum
		if {$lnum > 20000} {
		    return "Read too many lines. Something is wrong!"
		}
		# scan for the "START" line	    
		if $scan {
		    if {[string first START $line] != -1} {
			set scan 0
			# get the file name
			foreach w $line {
			    if {[string first FILE= $w] != -1} {
				set l [string first \] $w]
				incr l
				set filename [string range $w $l end]
			    }
			}
			set l [expr 8 + [string first VARY= $line]]
			set title [string trim [string range $line $l end]]
		    }
		} elseif {[set l [string first DATA= $line]] != -1} {
		    incr l 5
		    # change commas to spaces
		    regsub -all {,} [string range $line $l end] { } data
		    set j 0
		    set i 1
		    # anything left to read?
		    while {[lindex $data $j] != ""} {
			lappend pt($i) [lindex $data $j] 
			incr i
			if {$i > $ndet} {set i 1}
			incr j
		    }
		    set l [lsearch $line A4=]
		    if {$l == -1} {
			return "No A4= value"
		    }
		    incr l
		    lappend angles [lindex $line $l]
		}
	    }
	    catch {close $fl}
	    
	    # BT1 normalization parameters
	    set norm_list { null_value \
		    2.700  2.479  2.827  2.483  2.260  2.347  2.011  1.749  \
		    1.630  1.360  1.339  1.218  1.058  1.000  1.054  0.953  \
		    0.941  0.985  1.031  1.021  0.982  1.011  0.900  1.118  \
		    0.955  1.056  0.973  0.974  0.943  0.877  0.872  0.820  }
	    set zero_list { null_value \
		    0.00   1.00   1.29  -0.48   1.53  -0.98   2.03   0.89   \
		    1.54   1.28   0.40   0.35   1.53  -1.57   0.63   1.43   \
		    -0.08  -0.01  -0.78   0.16  -1.08  -2.08  -1.23  -0.47   \
		    0.43  -0.27  -2.60   0.88  -1.34   2.24   3.00   4.00   }
		
	    set bank 0
	    # is this old or new BT1 data?
	    if {$ndet == 5} {set bankoffset 20} {set bankoffset 5}
	    set ptsread 0
	    while {$bank < $ndet} {
		incr bank
		set data [file root [file tail $filename]]_$bank 
		initdata $data
		global $data
		set ${data}(xlabel) "2theta"
		set ${data}(xunits) "2theta"
		set ${data}(ylabel) "counts"
		set ${data}(title) "Bank $bank $title"
		set zero 0
		if {$zerocorr} {set zero [lindex $zero_list $bank]}
		set mult 1.
		if {$multcorr} {set mult [lindex $norm_list $bank]}
		set ptnum 0
		foreach count $pt($bank) angle $angles {
		    if {$count > 0} {
			lappend ${data}(x) \
				[expr $angle + ($bank-1.)*$bankoffset - $zero/100.]
			lappend ${data}(y) [expr $mult*$count]
			lappend ${data}(esd) [expr $mult*sqrt($count)]
			incr ptsread
			incr ptnum
		    }
		}
		resetdata $data
		lappend graph(plotlist) $data
	    }
	} errmsg] {
	    # try to process the last data we have read
	    catch {resetdata $data}
	    return "BT1 Active error. Error reading line $lnum: $errmsg"
	}
	catch {close $fp} test
	puts "$ptsread non-zero points read"
	return {}
    }
}
